searchData={"items":[{"type":"module","title":"LiveViewNative","doc":"A module providing supporting functions for LiveView Native.","ref":"LiveViewNative.html"},{"type":"function","title":"LiveViewNative.platform/1","doc":"Returns an environment struct for a LiveView Native platform given its\n`platform_id` or `:error` if not found.\n\nUsed to introspect platforms at compile-time or runtime.","ref":"LiveViewNative.html#platform/1"},{"type":"function","title":"LiveViewNative.platform!/1","doc":"Returns an environment struct for a LiveView Native platform given its\n`platform_id` or raises if not found.\n\nSame as `platform/1` but raises `RuntimeError` instead of returning\n`:error` if no platform exists for the given `platform_id`","ref":"LiveViewNative.html#platform!/1"},{"type":"function","title":"LiveViewNative.platforms/0","doc":"Returns a list of environment structs for all LiveView Native platforms.","ref":"LiveViewNative.html#platforms/0"},{"type":"module","title":"LiveViewNative.Component","doc":"Upgrades a Phoenix Component to a native Phoenix Component.\n\nTo use, inherit with `use LiveViewNative.Component`\nlike so:\n\nExample:\n\n```\ndefmodule MyApp.MyComponent do\n  use MyAppWeb, :live_component\n  use LiveViewNative.Component\n\n  # ...\nend\n```","ref":"LiveViewNative.Component.html"},{"type":"module","title":"LiveViewNative.LiveComponent","doc":"Upgrades a Live Component to a native Live Component.\n\nTo use, inherit with `use LiveViewNative.LiveComponent`\nlike so:\n\n```\ndefmodule MyApp.MyComponent do\n  use MyAppWeb, :live_component\n  use LiveViewNative.LiveComponent\n\n  # ...\nend\n```","ref":"LiveViewNative.LiveComponent.html"},{"type":"module","title":"LiveViewNative.LiveSession","doc":"Conducts platform detection on socket connections and applies\nnative assigns.","ref":"LiveViewNative.LiveSession.html"},{"type":"function","title":"LiveViewNative.LiveSession.on_mount/4","doc":"","ref":"LiveViewNative.LiveSession.html#on_mount/4"},{"type":"module","title":"LiveViewNative.LiveView","doc":"Upgrades a LiveView to a native LiveView.\n\nTo use, inherit with `use LiveViewNative.LiveView`\nlike so:\n\nExample:\n\n```\ndefmodule MyApp.MyLive do\n  use MyAppWeb, :live_view\n  use LiveViewNative.LiveView\n\n  # ...\nend\n```","ref":"LiveViewNative.LiveView.html"},{"type":"module","title":"LiveViewNative.TagEngine","doc":"An implementation of `Phoenix.LiveView.TagEngine` that omits\nHTML-centric template rules.","ref":"LiveViewNative.TagEngine.html"},{"type":"function","title":"LiveViewNative.TagEngine.classify_type/1","doc":"","ref":"LiveViewNative.TagEngine.html#classify_type/1"},{"type":"function","title":"LiveViewNative.TagEngine.void?/1","doc":"","ref":"LiveViewNative.TagEngine.html#void?/1"},{"type":"module","title":"LiveViewNative.Templates","doc":"Provides functionality for preprocessing LiveView Native\ntemplates.","ref":"LiveViewNative.Templates.html"},{"type":"function","title":"LiveViewNative.Templates.precompile/1","doc":"","ref":"LiveViewNative.Templates.html#precompile/1"},{"type":"task","title":"mix lvn.install","doc":"Installer Mix task for LiveView Native: `mix lvn.install`","ref":"Mix.Tasks.Lvn.Install.html"},{"type":"function","title":"Mix.Tasks.Lvn.Install.run/1","doc":"","ref":"Mix.Tasks.Lvn.Install.html#run/1"},{"type":"extras","title":"Overview","doc":"# Overview\n\nLiveView Native is a framework for building native applications using Elixir and Phoenix LiveView. It allows a single application to serve a multitude of clients by transforming platform-specific template code into native user interfaces. Here's a basic example that serves web, iOS, iPadOS and macOS clients natively:\n\n \n\n#","ref":"overview.html"},{"type":"extras","title":"Source - Overview","doc":"```elixir\n# lib/my_app_web/live/hello_live.ex\ndefmodule MyAppWeb.HelloLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  @impl true\n  def render(%{platform_id: :swiftui} = assigns) do\n    # This UI renders on the iPhone / iPad app\n    ~SWIFTUI\"\"\"\n     \n       \n        Hello native!\n       \n     \n    \"\"\"\n  end\n\n  @impl true\n  def render(%{} = assigns) do\n    # This UI renders on the web\n    ~H\"\"\"\n     \n       \n        Hello web!\n       \n     \n    \"\"\"\n  end\nend\n```\n\n#","ref":"overview.html#source"},{"type":"extras","title":"iOS - Overview","doc":"![Hello World - iOS](./assets/images/hello-iphone.png)\n\n#","ref":"overview.html#ios"},{"type":"extras","title":"iPadOS - Overview","doc":"![Hello World - iPadOS](./assets/images/hello-ipad.png)\n\n#","ref":"overview.html#ipados"},{"type":"extras","title":"macOS - Overview","doc":"![Hello World - macOS](./assets/images/hello-mac.png)\n\n#","ref":"overview.html#macos"},{"type":"extras","title":"Web - Overview","doc":"![Hello World - Web](./assets/images/hello-web.png)\n\n \n\nBy using LiveView Native in an existing Phoenix project, developers are able to deliver rich, real-time UIs for a multitude of web and non-web clients generated entirely by the server. Live sessions, state, event callbacks and glue code can be shared across all target platforms, with each platform having its own custom-tailored template or function component.\n\nLiveView Native officially supports using LiveView for the following native clients:\n\n- iOS 16+\n- macOS 13+\n- watchOS 9+\n- Android\n\nLiveView Native requires some foundational knowledge to use. You should already be familiar with [Elixir](https://elixir-lang.org/), the [Phoenix Framework](https://www.phoenixframework.org/) and [Phoenix LiveView](https://github.com/phoenixframework/phoenix_live_view). If you're looking to learn more about any of these subjects, there are a lot of great resources available. Some recommended materials include the [Elixir guides](https://elixir-lang.org/getting-started/introduction.html), [Elixir learning resources page](https://elixir-lang.org/learning.html), [Phoenix guides](https://hexdocs.pm/phoenix/overview.html), [Phoenix community page](https://hexdocs.pm/phoenix/community.html) and the [Phoenix LiveView HexDocs](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html). \n\nWith those prerequisites out of the way, [let's get LiveView Native installed](./installation.md)!","ref":"overview.html#web"},{"type":"extras","title":"Installation","doc":"# Installation\n\nThere are a few steps that must be completed before you can use LiveView Native. This document covers them, which are as follows:\n\n1. Setup a Phoenix project with the minimum required versions of Elixir, Phoenix and LiveView.\n2. Add LiveView Native to your `mix.exs` dependencies.\n3. Add any number of platform libraries to enable support for native clients.\n4. Fetch dependencies.\n5. Enable LiveView Native within your app.","ref":"installation.html"},{"type":"extras","title":"1. Prepare your Phoenix app - Installation","doc":"To use LiveView Native, you must have an existing Phoenix project. If you don't have one, follow the [Up and Running](https://hexdocs.pm/phoenix/up_and_running.html) section of the official Phoenix guide to create one.\n\nNext, make sure your project meets the following requirements:\n\n- it uses **Elixir 1.15 or greater** — to enforce this, update your `mix.exs`:\n```elixir\ndef project do\n  [\n    # ...\n    elixir: \"~> 1.15\",\n    # ...\n  ]\nend\n```\n\n- it uses **Phoenix >= 1.7** and **Phoenix LiveView >= 0.18** — your `mix.exs` should partially resemble this:\n```elixir\ndef deps do\n  [\n    {:phoenix, \"~> 1.7\"},\n    {:phoenix_live_view, \"~> 0.18\"},\n    # other dependencies here...\n  ]\nend\n```\n\nIf your project doesn't meet this criteria then it will need to be upgraded. Some helpful resources for this include the [Elixir installation page](https://elixir-lang.org/install.html), [Phoenix installation page](https://hexdocs.pm/phoenix/installation.html), as well as the changelogs for [Elixir](https://github.com/elixir-lang/elixir/blob/main/CHANGELOG.md), [Phoenix](https://github.com/phoenixframework/phoenix/blob/main/CHANGELOG.md) and [Phoenix LiveView](https://github.com/phoenixframework/phoenix_live_view/blob/main/CHANGELOG.md).","ref":"installation.html#1-prepare-your-phoenix-app"},{"type":"extras","title":"2. Add LiveView Native - Installation","doc":"Once you've met the requirements to use LiveView Native, simply add it to your list of dependencies in your project's mix.exs:\n\n```elixir\ndef deps do\n  [\n    # other dependencies here...\n    {:live_view_native, \"~> 0.1\"}\n  ]\nend\n```","ref":"installation.html#2-add-liveview-native"},{"type":"extras","title":"3. Add platform libraries - Installation","doc":"The `:live_view_native` dependency isn't useful on its own. You'll also need to add any _platform libraries_ you want your project to be compatible with. Platform libraries provide the native implementations that allow those platforms' clients to connect to your app and render LiveView paths as native user interfaces.\n\nThis guide only covers installation for the officially supported platforms, [SwiftUI](https://hexdocs.pm/live_view_native_swift_ui) (iOS, macOS and watchOS) and [Jetpack](https://hexdocs.pm/live_view_native_jetpack) (Android). For information on using a third-party platform, consult that library's documentation. \n\n \n\n#","ref":"installation.html#3-add-platform-libraries"},{"type":"extras","title":"SwiftUI - Installation","doc":"Adds compatibility for iOS 16+, macOS 13+ and watchOS 9+.\n\n```elixir\ndef deps do\n  [\n    # other dependencies here...\n    {:live_view_native_swift_ui, \"~> 0.1\"}\n  ]\nend\n```\n\n#","ref":"installation.html#swiftui"},{"type":"extras","title":"Jetpack - Installation","doc":"Adds compatibility for Android.\n\n> ##","ref":"installation.html#jetpack"},{"type":"extras","title":"Warning {: .warning} - Installation","doc":"> The Jetpack client is under development and is not yet usable in end-user applications.\n> For more information, check the repo for `live_view_native_jetpack` [here](https://github.com/liveview-native/liveview-client-jetpack).\n\n```elixir\ndef deps do\n  [\n    # other dependencies here...\n    {:live_view_native_jetpack, \"~> 0.0.0\"}\n  ]\nend\n```","ref":"installation.html#warning-warning"},{"type":"extras","title":"4. Fetch dependencies - Installation","doc":"Next, fetch any new dependencies you added to your `mix.exs`.\n\n```bash\nmix deps.get\n```","ref":"installation.html#4-fetch-dependencies"},{"type":"extras","title":"5. Enable LiveView Native - Installation","doc":"LiveView Native includes a Mix task that can automatically handle the process of configuring your project to support it. If that is not to your liking, manual setup is also an option. This guide includes instructions for both of these approaches.\n\n \n\n#","ref":"installation.html#5-enable-liveview-native"},{"type":"extras","title":"Automatic - Installation","doc":"Within your project directory, run the following command:\n\n```bash\nmix lvn.install\n```\n\nThis command will prompt you to answer a few questions. If everything goes well, you should see a message that your project has been configured to use LiveView Native.\n\n#","ref":"installation.html#automatic"},{"type":"extras","title":"Manual - Installation","doc":"After adding the `:live_view_native` Hex package and any platform libraries, define a key-value configuration in your [Config](https://hexdocs.pm/elixir/main/Config.html) file (this is typically found in `config/config.exs`). This configuration expects a `:plugins` option which takes a list of any platform libraries you want your application to support. Platform libraries are represented by their top-level namespace module:\n\n```elixir\n# config.exs\n\n# Use LiveView Native to add support for native platforms\nconfig :live_view_native,\n  plugins: [\n    # other plugins here...\n    LiveViewNativeSwiftUi,\n    LiveViewNativeJetpack\n  ]\n```\n\nNext, create a project for each platform's native client using the official tools provided by each platform.","ref":"installation.html#manual"},{"type":"extras","title":"Post-Installation - Installation","doc":"Once LiveView Native is installed and your application is properly configured, you should be able to run your\nPhoenix app in development as usual:\n\n```bash\niex -S mix phx.server\n```\n\nTo confirm that LiveView Native has been properly installed, call the following function in an IEx\nsession. It should return a map describing all of the native platform libraries that have been installed\nfor your app.\n\n```elixir\niex(1)> LiveViewNative.platforms()\n```\n\nConfirm that each platform listed (excluding `web`) has a platform-specific client app (i.e. Xcode,\nAndroid Studio, etc.) for connecting to your LiveView Native backend. If you used `mix lvn.install`\nto enable LiveView Native, these project files will be placed in the `native/` directory of your app.  \n\nIf everything looks good, continue on to writing [your first native LiveView](./your-first-native-liveview.md).","ref":"installation.html#post-installation"},{"type":"extras","title":"Your First Native LiveView","doc":"# Your First Native LiveView\n\nNow that you've [installed LiveView Native](./installation.md), its time to create a LiveView\nthat uses it. This guide explains how to create a basic LiveView module that can serve different\ntemplates to both web and native platforms while sharing non-template code between them.","ref":"your-first-native-liveview.html"},{"type":"extras","title":"A basic LiveView - Your First Native LiveView","doc":"> ##","ref":"your-first-native-liveview.html#a-basic-liveview"},{"type":"extras","title":"Info {: .info} - Your First Native LiveView","doc":"> For more information on LiveView in general, see the docs for `Phoenix.LiveView` [here](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html).\n\nTo begin, create a new LiveView module and add it to your Phoenix router. The path for this LiveView\nshould be identical to the path you chose when setting up your native client app (this defaults to `\"/\"`,\nwhen using `mix lvn.install` to install LiveView Native).\n\nFirst create a new LiveView somewhere in your app that inherits `LiveViewNative.LiveView`, like so:\n\n```elixir\n# lib/my_app_web/live/hello_live.ex\ndefmodule MyAppWeb.HelloLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  @impl true\n  def render(%{} = assigns) do\n    # This UI renders on the web\n    ~H\"\"\"\n     \n       \n        Hello web!\n       \n     \n    \"\"\"\n  end\nend\n```\n\nThen, add it to your app's router:\n\n```elixir\n# lib/my_app_web/router.ex\ndefmodule MyAppWeb.Router do\n  use MyAppWeb, :router\n\n  #","ref":"your-first-native-liveview.html#info-info"},{"type":"extras","title":"Pipelines, etc. - Your First Native LiveView","doc":"scope \"/\", MyAppWeb do\n    pipe_through :browser\n\n    #","ref":"your-first-native-liveview.html#pipelines-etc"},{"type":"extras","title":"Other routes... - Your First Native LiveView","doc":"live \"/\", HelloLive\n  end\n\n  #","ref":"your-first-native-liveview.html#other-routes"},{"type":"extras","title":"Routes for LiveDashboard, Swoosh, etc. - Your First Native LiveView","doc":"end\n```\n\nThen, run your app and navigate to your local development server in a web browser.\nYou should see the LiveView you created.\n\n![Hello World - Web](./assets/images/hello-web.png)","ref":"your-first-native-liveview.html#routes-for-livedashboard-swoosh-etc"},{"type":"extras","title":"A native LiveView - Your First Native LiveView","doc":"To support non-web platforms using LiveView Native, simply provide a template for each\nplatform you want to support. This is done by extending LiveView's standard `render/1` callback.\n\nEach platform will have its own custom `:platform_id`, render sigil and HEEx syntax, which are\nall pulled in by the `LiveViewNative.LiveView` macro inherited toward the top of your LiveView.\nThe following example demonstrates handling the `:swiftui` platform for LiveView Native apps\nwith the `:live_view_native_swift_ui` dependency, which covers iOS, iPadOS and macOS support:\n\n```elixir\n# lib/my_app_web/live/hello_live.ex\ndefmodule MyAppWeb.HelloLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  @impl true\n  def render(%{platform_id: :swiftui} = assigns) do\n    # This UI renders on the iPhone / iPad app\n    ~SWIFTUI\"\"\"\n     \n       \n        Hello native!\n       \n     \n    \"\"\"\n  end\n\n  @impl true\n  def render(%{} = assigns) do\n    # This UI renders on the web\n    ~H\"\"\"\n     \n       \n        Hello web!\n       \n     \n    \"\"\"\n  end\nend\n```\n\nLoading the native project at `native/swiftui/MyApp/MyApp.xcodeproj` and running it in\nSimulator will render the native template in various devices for the SwiftUI platform:\n\n \n\n#","ref":"your-first-native-liveview.html#a-native-liveview"},{"type":"extras","title":"iOS - Your First Native LiveView","doc":"![Hello World - iOS](./assets/images/hello-iphone.png)\n\n#","ref":"your-first-native-liveview.html#ios"},{"type":"extras","title":"iPadOS - Your First Native LiveView","doc":"![Hello World - iPadOS](./assets/images/hello-ipad.png)\n\n#","ref":"your-first-native-liveview.html#ipados"},{"type":"extras","title":"macOS - Your First Native LiveView","doc":"![Hello World - macOS](./assets/images/hello-mac.png)\n\n#","ref":"your-first-native-liveview.html#macos"},{"type":"extras","title":"Web - Your First Native LiveView","doc":"![Hello World - Web](./assets/images/hello-web.png)\n\n \n\nIf everything looks as it should, congratulations! Your app is now using LiveView Native to\nserve multiple platform-specific templates to both web and non-web clients.","ref":"your-first-native-liveview.html#web"},{"type":"extras","title":"What's next? - Your First Native LiveView","doc":"If you've reached this section of the guide, you should have a cursory understanding of LiveView\nNative, how to install it in an existing Phoenix project and how to write a basic \"Hello World\"\nLiveView that renders a different UI for web and non-web clients.\n\nTo use LiveView Native to its full potential, you will need to dig deeper into the following subjects:\n\n1. LiveView Native features and concepts\n2. Additional libraries for the platform(s) your app supports\n3. Building UIs for the platform(s) your app support\n\nThis guide will cover the first point in the pages to follow. For the other two, you will need to consult the\nrelevant documentation, guides and supporting literature for the platform(s) you want your app to serve. Below\nare some recommended resources for the SwiftUI and Jetpack platforms:\n\n \n\n#","ref":"your-first-native-liveview.html#what-s-next"},{"type":"extras","title":"SwiftUI - Your First Native LiveView","doc":"Covers iOS 16+, macOS 13+ and watchOS 9+.\n\n- [Platform library HexDocs](https://hexdocs.pm/live_view_native_swift_ui/)\n- [Swift library docs](https://liveview-native.github.io/liveview-client-swiftui/documentation/liveviewnative/)\n- [SwiftUI docs](https://developer.apple.com/documentation/swiftui/)\n- [SwiftUI tutorials](https://developer.apple.com/tutorials/swiftui)\n- [Xcode docs](https://developer.apple.com/documentation/xcode)\n\n#","ref":"your-first-native-liveview.html#swiftui"},{"type":"extras","title":"Jetpack - Your First Native LiveView","doc":"Covers Android.\n\n- [Platform library HexDocs](https://hexdocs.pm/live_view_native_jetpack/)\n- [Jetpack Compose tutorial](https://developer.android.com/jetpack/compose/tutorial)","ref":"your-first-native-liveview.html#jetpack"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting\n\nSometimes you might get blocked by a compiler error or other issue during installation. If this happens, you might try clearing your build directory and recompiling from a clean state. To do this, run the following commands from your project's root directory:\n\n```bash\nrm -rf _build\nrm -rf deps\nmix deps.get\nmix compile\n```\n\nThis should ideally resolve without error. If it doesn't, feel free to [submit an issue](https://github.com/liveview-native/live_view_native/issues/new) on the GitHub repo for this library or ask in the `#liveview-native` channel of [Elixir Slack](https://elixir-lang.slack.com/); a member of the LiveView Native core team or other community member should be able to help you troubleshoot the problem.","ref":"troubleshooting.html"},{"type":"extras","title":"Template Syntax","doc":"# Template Syntax\n\nLiveView Native platforms use EEx templates which resemble the native APIs they're based\non as closely as possible. This is in contrast to frameworks that provide custom element\nnames which then get translated to native components.\n\nConsider the following example for a SwiftUI view. It is represented in LiveView Native using\nall of the same elements like `List`, `HStack`, etc. To do this we adapt the original SwiftUI\nelement and attribute names to the semantics of the EEx syntax using `~SWIFTUI`, like so:\n\n \n\n#","ref":"template-syntax.html"},{"type":"extras","title":"Elixir - Template Syntax","doc":"LiveView Native components for SwiftUI use SwiftUI-like syntax.\n\n```elixir\ndefmodule MyAppWeb.MyComponents do\n  use Phoenix.LiveView\n  use LiveViewNative.Component\n\n  def album_detail(%{platform_id: :swiftui} = assigns) do\n    ~SWIFTUI\"\"\"\n     \n       \n         \n           \n           \n               \n               \n           \n         \n       \n     \n    \"\"\"\n  end\nend\n```\n\n#","ref":"template-syntax.html#elixir"},{"type":"extras","title":"Swift - Template Syntax","doc":"The same component would be represented in SwiftUI like this:\n\n```swift\nimport SwiftUI\n\nstruct AlbumDetail: View {\n  var album: Album\n\n  var body: some View {\n    List(album.songs) { song in\n      HStack {\n        Image(album.cover)\n        VStack(alignment: .leading) {\n          Text(song.title)\n            .fontWeight(.bold)\n          Text(song.artist.name)\n        }\n      }\n    }\n  }\n}\n```\n\n \n\nHere we can observe various semantic changes to port SwiftUI code to EEx:\n\n1. SwiftUI view names are used as elements, like [List](https://developer.apple.com/documentation/swiftui/list) and [VStack](https://developer.apple.com/documentation/swiftui/vstack).\n   - The argument syntax (i.e. `alignment: .leading` to `alignment=\"leading\"`) is adapted as well.\n2. The struct value `album` is an assign, `@album`, instead.\n3. Instead of passing `album.songs` as an argument, we use a comprehension.\n    - ` ` elements take their arguments as values, similar to HTML.\n4. We can't call a function on an element like in SwiftUI, so the `.fontWeight(.bold)` modifier function is called as `font_weight(:bold)` and passed to the `modifiers` attribute.\n\nThese conventions can generally be applied to all sorts of examples when using LiveView Native to build SwiftUI views.\nBecause LiveView Native is modular, each platform library will have its own way of \"bridging the gap\" between the Elixir\nside of your app and the native side.\n\nOfficially supported platform libraries are designed with general platform-parity in mind; while some light abstractions\nmay be used in cases where the native syntax doesn't map perfectly onto LiveView templates, any names of elements,\nattributes, structs, enums, types, etc. are typically carried over in identical or approximate (i.e. camel case to\nsnake case) form.\n\nFor more information on writing platform-specific template code, consult the documentation for each platform you want\nyour app to support.\n\n \n\n#","ref":"template-syntax.html#swift"},{"type":"extras","title":"SwiftUI - Template Syntax","doc":"Covers iOS 16+, macOS 13+ and watchOS 9+.\n\n- [Platform library HexDocs](https://hexdocs.pm/live_view_native_swift_ui/)\n- [Swift library docs](https://liveview-native.github.io/liveview-client-swiftui/documentation/liveviewnative/)\n- [SwiftUI docs](https://developer.apple.com/documentation/swiftui/)\n- [SwiftUI tutorials](https://developer.apple.com/tutorials/swiftui)\n- [Xcode docs](https://developer.apple.com/documentation/xcode)\n\n#","ref":"template-syntax.html#swiftui"},{"type":"extras","title":"Jetpack - Template Syntax","doc":"Covers Android.\n\n- [Platform library HexDocs](https://hexdocs.pm/live_view_native_jetpack/)\n- [Jetpack Compose tutorial](https://developer.android.com/jetpack/compose/tutorial)","ref":"template-syntax.html#jetpack"},{"type":"extras","title":"Modifiers","doc":"# Modifiers\n\nPlatform libraries may provide any number of modifiers for customizing the look and feel of native\nelements. These modifiers are automatically available as functions when rendering platform-specific\ntemplates in a module that uses one of the following macros:\n\n- `LiveViewNative.LiveView`\n- `LiveViewNative.LiveComponent`\n- `LiveViewNative.Component`\n\nModifiers are typically used to adjust styling (colors, typography, etc.), presentation, interactivity\nand other properties of the native UI you're targeting.","ref":"modifiers.html"},{"type":"extras","title":"Modifier functions - Modifiers","doc":"Here's a simple example of calling modifier functions inline for elements with the `modifiers` attribute in SwiftUI:\n\n \n\n#","ref":"modifiers.html#modifier-functions"},{"type":"extras","title":"Source - Modifiers","doc":"```elixir\ndefmodule MyAppWeb.ModifiersExampleLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  @impl true\n  def render(%{platform_id: :swiftui} = assigns) do\n    # This UI renders on the iPhone / iPad app\n    ~SWIFTUI\"\"\"\n     \n       This text is normal \n       This text is bold \n       \n       \n          foreground_style({:color, :white})\n        }>\n            foreground_style({:color, :red})\n          } />\n         \n       \n     \n    \"\"\"\n  end\nend\n```\n\n#","ref":"modifiers.html#source"},{"type":"extras","title":"Result - Modifiers","doc":"![Modifiers example](./assets/images/modifiers-example.png)\n\n \n\nModifier functions may have different arities and take different types of arguments, which are generally based on the\noriginal APIs they're based on. All modifier functions return a `%LiveViewNativePlatform.Env{}` struct (same as the\n`@native` assign) which can be passed to other modifier functions, effectively allowing them to be chained together.\n\nFor more information of which modifiers a platform supports and how to use them, check the documentation for that\nplatform as well as the relevant source material for that platform.","ref":"modifiers.html#result"},{"type":"extras","title":"Modifier Classes - Modifiers","doc":"Using a lot of modifiers in your templates can cause them to become overly verbose and difficult to maintain over\ntime. You also might want to share modifiers between many different views and elements instead of copying them\nacross templates. Modifier classes solve both of these problems by letting you decouple your modifiers from your\ntemplates.\n\nHere's the previous example, adjusted to use modifier classes defined in a separate module:\n\n \n\n#","ref":"modifiers.html#modifier-classes"},{"type":"extras","title":"modifiers_example_live.ex - Modifiers","doc":"```elixir\ndefmodule MyAppWeb.ModifiersExampleLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  import MyAppWeb.Modclasses, only: [modclass: 3]\n\n  @impl true\n  def render(%{platform_id: :swiftui} = assigns) do\n    ~SWIFTUI\"\"\"\n     \n       This text is normal \n       This text is bold \n       \n       \n         \n           \n         \n       \n     \n    \"\"\"\n  end\nend\n```\n\n#","ref":"modifiers.html#modifiers_example_live-ex"},{"type":"extras","title":"modclasses.ex - Modifiers","doc":"```elixir\ndefmodule MyAppWeb.Modclasses do\n  use LiveViewNative.Modclasses, platform: :swiftui\n\n  def modclass(native, \"bold\", _assigns) do\n    font_weight(native, :bold)\n  end\n\n  def modclass(native, \"spacer\", _assigns) do\n    frame(native, height: 16)\n  end\n\n  def modclass(native, \"heart\", _assigns) do\n    native\n    |> background(alignment: :center, content: :heart_bg)\n    |> foreground_style({:color, :white})\n  end\n\n  def modclass(native, \"heart_bg\", _assigns) do\n    native\n    |> frame(width: 32, height: 32)\n    |> foreground_style({:color, :red})\n  end\nend\n```\n\n#","ref":"modifiers.html#modclasses-ex"},{"type":"extras","title":"Result - Modifiers","doc":"![Modifiers example](./assets/images/modifiers-example.png)\n\n \n\nAn element can have any number of modifier classes, providing some composability for modifier functions:\n\n \n\n#","ref":"modifiers.html#result"},{"type":"extras","title":"modifiers_example_live.ex - Modifiers","doc":"```elixir\ndefmodule MyAppWeb.ModifiersExampleLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  import MyAppWeb.Modclasses, only: [modclass: 3]\n\n  @impl true\n  def render(%{platform_id: :swiftui} = assigns) do\n    ~SWIFTUI\"\"\"\n     \n       This text is normal \n       This text is bold \n       This text is bold \n       This text is bold and italic \n     \n    \"\"\"\n  end\nend\n```\n\n#","ref":"modifiers.html#modifiers_example_live-ex"},{"type":"extras","title":"modclasses.ex - Modifiers","doc":"```elixir\ndefmodule MyAppWeb.Modclasses do\n  use LiveViewNative.Modclasses, platform: :swiftui\n\n  def modclass(native, \"bold\", _assigns) do\n    font_weight(native, :bold)\n  end\n\n  def modclass(native, \"italic\", _assigns) do\n    italic(native, %{})\n  end\nend\n```\n\n#","ref":"modifiers.html#modclasses-ex"},{"type":"extras","title":"Result - Modifiers","doc":"![Modclasses example](./assets/images/modclasses-example.png)\n\n \n\nModclasses within templates are translated at compile-time to their inline counterparts, so passing an assign or other\ndynamic value to the `modclass` attribute won't work. To support dynamic modifier classes that reference assigns or the\nmodifier name itself, define any conditional logic within `modclass/3`.","ref":"modifiers.html#result"},{"type":"extras","title":"Render Patterns","doc":"# Render Patterns\n\nThere are multiple ways to render native UIs with LiveView Native. This document covers various\npatterns and when you might use them.\n\n## `render/1` function clauses\n\nThe most common pattern that is often used throughout this guide is the function clause pattern.\nEach function clause matches on the `:platform_id`, allowing each platform to define its own UI:\n\n```elixir\n# lib/my_app_web/live/hello_live.ex\ndefmodule MyAppWeb.HelloLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  @impl true\n  def render(%{platform_id: :swiftui} = assigns) do\n    # This UI renders on the iPhone / iPad app\n    ~SWIFTUI\"\"\"\n     \n       \n        Hello native!\n       \n     \n    \"\"\"\n  end\n\n  @impl true\n  def render(%{} = assigns) do\n    # This UI renders on the web\n    ~H\"\"\"\n     \n       \n        Hello web!\n       \n     \n    \"\"\"\n  end\nend\n```","ref":"render-patterns.html"},{"type":"extras","title":"Function components - Render Patterns","doc":"Platform-specific components can also be defined. For example:\n\n```elixir\n# lib/my_app_web/live/hello_live.ex\ndefmodule MyAppWeb.SharedComponents do\n  use Phoenix.Component\n  use LiveViewNative.Component\n\n  import ElixirconfChatWeb.Modclasses.SwiftUi, only: [modclass: 3]\n\n  def logo(%{platform_id: :swiftui} = assigns) do\n    ~SWIFTUI\"\"\"\n     \n       \n         \n     \n    \"\"\"\n  end\n\n  def logo(%{} = assigns) do\n    ~H\"\"\"\n     \n       \n         \n     \n    \"\"\"\n  end\nend\n```","ref":"render-patterns.html#function-components"},{"type":"extras","title":"External template files - Render Patterns","doc":"If you would prefer to break your render function out into separate template files, you can\nconditionally render platform-specific templates using the `render_native/1` macro. External\ntemplate files are namespaced according to their `:platform_id`:\n\n \n\n#","ref":"render-patterns.html#external-template-files"},{"type":"extras","title":"hello_live.ex - Render Patterns","doc":"```elixir\ndefmodule MyAppWeb.HelloLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  @impl true\n  def render(%{} = assigns) do\n    render_native(assigns)\n  end\nend\n```\n\n#","ref":"render-patterns.html#hello_live-ex"},{"type":"extras","title":"hello_live.html.heex - Render Patterns","doc":"```heex\n \n   \n     A web template, courtesy of hello_live.html.heex \n   \n \n```\n\n#","ref":"render-patterns.html#hello_live-html-heex"},{"type":"extras","title":"hello_live.swiftui.heex - Render Patterns","doc":"```heex\n \n   \n     A SwiftUI template, courtesy of hello_live.swiftui.heex \n   \n \n```","ref":"render-patterns.html#hello_live-swiftui-heex"},{"type":"extras","title":"Targeting specific devices - Render Patterns","doc":"Conditional rendering based on device type is also supported, depending on the platform library you're using.\nThe following example for the SwiftUI platform renders different text on various devices:\n\n```elixir\n# lib/my_app_web/live/hello_live.ex\ndefmodule MyAppWeb.SharedComponents do\n  use Phoenix.Component\n  use LiveViewNative.Component\n\n  import ElixirconfChatWeb.Modclasses.SwiftUi, only: [modclass: 3]\n\n  def logo(%{platform_id: :swiftui} = assigns) do\n    ~SWIFTUI\"\"\"\n     \n       \n          %>\n           Hello macOS! \n          %>\n           Hello iPadOS! \n          %>\n           Hello watchOS! \n          %>\n           Hello tvOS! \n          %>\n           Hello iOS! \n       \n     \n    \"\"\"\n  end\nend\n```","ref":"render-patterns.html#targeting-specific-devices"},{"type":"extras","title":"Handling Events","doc":"# Handling Events\n\nEvents are a common part of any LiveView application. On the server, LiveView Native handles events identically to LiveView\non the web. How the client handles events depends on the native platform and its respective client implementation. This\ndocument provides some general information on events regardless of platform.","ref":"handling-events.html"},{"type":"extras","title":"Translated Bindings - Handling Events","doc":"Platforms may translate [Phoenix bindings](https://hexdocs.pm/phoenix_live_view/bindings.html) to their native counterparts\nwhere it makes sense. The most common example of this is `phx-click` which listens for tap events on SwiftUI and Jetpack\ntargets.\n\nOn the server, events are handled using standard LiveView callbacks like `handle_event/3` regardless of what platform they\ncome from. Callbacks can also be shared across platforms, assuming the events and their params are compatible. Here is a\nbasic example of sharing a `handle_event/3` callback in this way:\n\n```elixir\n# hello_live.ex\ndefmodule MyAppWeb.HelloLive do\n  use Phoenix.LiveView\n  use LiveViewNative.LiveView\n\n  @impl true\n  def mount(_params, _session, socket) do\n    {:ok, assign(socket, name: \"World\")}\n  end\n\n  @impl true\n  def render(%{platform_id: :swiftui} = assigns) do\n    # This UI renders on the iPhone / iPad app\n    ~SWIFTUI\"\"\"\n     \n       Hello  ! \n     \n    \"\"\"\n  end\n\n  @impl true\n  def render(%{} = assigns) do\n    # This UI renders on the web\n    ~H\"\"\"\n     \n       \n        Hello  !\n       \n     \n    \"\"\"\n  end\n\n  @impl true\n  def handle_event(\"click_test\", _params, socket) do\n    # This event can be called from both SwiftUI and the web\n    {:noreply, assign(socket, name: \"José\")}\n  end\nend\n```\n\nIn this example, clicking the \"Hello World\" button changes it to say \"Hello José\" on both the web and iOS.\nThe same `handle_event/3` callback is fired in both cases.","ref":"handling-events.html#translated-bindings"},{"type":"extras","title":"Change Events - Handling Events","doc":"In LiveView Native, elements can support client-side changes to their value outside of a ` `.\nSynchronizing the values must be handled manually by the LiveView using change events.\n\n#","ref":"handling-events.html#change-events"},{"type":"extras","title":"Client-side changes - Handling Events","doc":"Use the `phx-change` attribute to respond to client-side changes to an element's value.\n\n```html\n \n```\n\nA [`handle_event/3`](`c:Phoenix.LiveView.handle_event/3`) implementation with the name `\"value-changed\"` will be called anytime the user changes the text of the `TextField`.\n\n```elixir\ndef handle_event(\"value-changed\", new_value, socket) do\n  {:noreply, assign(socket, text: new_value)}\nend\n```\n\nThe `phx-debounce`, `phx-throttle`, and `phx-target` attributes can be used to configure the event.\n\nRefer to the documentation for an element to find out if it supports client-side changes.\n\n#","ref":"handling-events.html#client-side-changes"},{"type":"extras","title":"Server-side changes - Handling Events","doc":"Each element has an attribute that controls its value.\nFor example, `TextField` in the SwiftUI client uses the attribute `text`.\nSee the documentation for an element to find out what attribute it uses.\n\nWhenever the attribute's value is changed, the client will update to display the new value.\nNo change event is sent when the server updates the value.\n\n#","ref":"handling-events.html#server-side-changes"},{"type":"extras","title":"Modifier change events - Handling Events","doc":"Some modifiers have values that can be changed by the client.\n\nTo receive change events on these modifiers, pass a `LiveViewNativePlatform.Modifier.Types.Event` to the `change` argument.\n\n```elixir\nsheet(is_presented: @show, change: \"presentation-changed\")\n```\n\nProvide a map to `change` for more advanced configuration.\n\n```elixir\nsheet(is_presented: @show, change: %{ event: \"presentation-changed\", debounce: 2000, target: @myself })\n```\n\nA change event will be sent with every argument of the modifier that can be change-tracked.\n\n```elixir\ndef handle_event(\"presentation-changed\", %{ \"is_presented\" => show }, socket) do\n  {:noreply, assign(socket, show: show)}\nend\n```\n\nIn the example above, the server can show/hide the sheet by setting the `show` assign.\nWhen the user swipes down on the sheet to close it, the `\"presentation-changed\"` event will be called with the value `false`.\n\nSee the documentation for a modifier to find out which arguments can be change-tracked.","ref":"handling-events.html#modifier-change-events"}],"content_type":"text/markdown"}